﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;


namespace PinscapeConfigTool
{
    public partial class Form2 : Form
    {
        public Form2(DeviceInfo dev)
        {
            InitializeComponent();
            this.dev = dev;
        }

        private void Form2_Load(object sender, EventArgs e)
        {
            Text = "Settings for " + dev;
            txtUnitNo.Text = String.Format("{0}", dev.getUnitNo());
            ckEnablePlunger.Checked = dev.plungerEnabled;
        }

        private DeviceInfo dev;

        private int readUnitNo()
        {
            int unitNo;
            if (Int32.TryParse(txtUnitNo.Text, out unitNo))
                return unitNo;
            else
                return dev.getUnitNo();
        }
        
        private bool HasDiffs()
        {
            return (ckEnablePlunger.Checked != dev.plungerEnabled
                    || readUnitNo() != dev.getUnitNo());
        }
        
        private void btnCancel_Click(object sender, EventArgs e)
        {
            ActiveForm.Close();
        }

        private void ckEnablePlunger_CheckedChanged(object sender, EventArgs e)
        {
            UpdateApply();
        }

        private void txtUnitNo_TextChanged(object sender, EventArgs e)
        {
            UpdateApply();
        }

        private void UpdateApply()
        {
            btnApply.Enabled = HasDiffs();
        }

        private void btnApply_Click(object sender, EventArgs e)
        {
            int u = readUnitNo();
            bool unitNoChanged = (u != dev.getUnitNo());

            if (u < 1 || u > 16)
            {
                MessageBox.Show("The unit number must be from 1 to 16.");
                return;
            }

            // Update the settings.  
            // Message prefix 0x41 0x01 = set configuration.
            // byte 3 -> new LedWiz unit number
            // byte 4 -> feature bit mask:
            //            0x01   -> enable CCD
            byte[] buf = new byte[9];
            buf[0] = 0x00;   // report ID - always 0
            buf[1] = 0x41;   // 0x41 -> pinscape special request
            buf[2] = 0x01;   // 0x01 -> request type = set config options
            buf[3] = (byte)(u - 1);   
            buf[4] = (ckEnablePlunger.Checked ? (byte)0x01 : (byte)0x00);
            if (dev.WriteUSB(buf))
            {
                // update our internal settings
                dev.plungerEnabled = ckEnablePlunger.Checked;

                // update the apply button
                UpdateApply();

                // warn about unit number changes
                if (unitNoChanged)
                {
                    MessageBox.Show("The unit number change won't take effect until you "
                        + "reboot the controller. Simply unplug all USB cables from the "
                        + "controller for a few moments to cut power, then plug it back in. "
                        + "After plugging it back in, click the Refresh button to update "
                        + "the device list.");

                    ActiveForm.Close();
                }
            }
        }

        private void label2_Click(object sender, EventArgs e)
        {

        }
    }
}
